/**
 * Copyright (c) 2002 Bright Side Factory.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Bright Side Factory (http://www.bs-factory.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Bright Side", "BS Factory" and "Bright Side Factory" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact info@bs-factory.org.
 *
 * 5. Products derived from this software may not be called "Bright Side",
 *    nor may "Bright Side" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Bright Side Factory.  For more
 * information on the Bright Side Factory, please see
 * <http://www.bs-factory.org/>.
 */

package com.bsf.lovDemo.views;

import com.bsf.lovDemo.util.help.HelpNode;
import com.bsf.lovDemo.util.help.HelpTreeModel;
import com.bsf.lovDemo.views.internalFrames.*;
import com.jgoodies.animation.Animation;
import com.jgoodies.animation.Animations;
import com.jgoodies.animation.Animator;
import com.jgoodies.animation.animations.BasicTextAnimation;
import com.jgoodies.animation.animations.BasicTextAnimations;
import com.jgoodies.animation.animations.GlyphAnimation;
import com.jgoodies.animation.components.BasicTextLabel;
import com.jgoodies.animation.components.GlyphLabel;
import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.swing.panels.SimpleInternalFrame;
import com.jgoodies.swing.util.UIFactory;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.util.ArrayList;

/**
 * This is the main page of the application. It creates the tool bar,
 * _quickStartIF, help navigation, viewer, dynamic help viewer, and status
 * bar.
 */

public final class MainPage extends JPanel {
    public static final int WELCOME_INTERNAL_FRAME = 0;
    public static final int LOV_IN_TABLE_INTERNAL_FRAME = 1;
    public static final int LOV_IN_DETAIL_INTERNAL_FRAME = 2;
    public static final int LOV_CUSTOMIZATION_INTERNAL_FRAME = 3;
    public static final int LOV_PERFORMANCES_INTERNAL_FRAME = 4;

    // SplitPanes
    private JSplitPane _mainSplitPane;
    private JSplitPane _leftSplitPane;
    private JSplitPane _rightSplitPane;

    // Internal frames
    private WelcomeInternalFrame _welcomeIF;
    private QuickStartInternalFrame _quickStartIF;
    private HelpContentInternalFrame _helpContentIF;
    private HelpNavigatorInternalFrame _helpNavigatorIF;
    private TableInternalFrame _lovTableInternalFrame;
    private ComboBoxesInternalFrame _lovComboBoxesInternalFrame;
    private PerformancesInternalFrame _performancesInternalFrame;
    private SqlCustomizationInternalFrame _sqlCustomizationInternalFrame;

    private GlyphLabel _statusGlyphLabel;
    private BasicTextLabel _firstStatusLabel;
    private BasicTextLabel _secondStatusLabel;
    private BasicTextLabel _animatedStatusLabel;

    // *************************************************************************
    // *************************** Public Methods ******************************
    // *************************************************************************

    public QuickStartInternalFrame getQuickStartInternalFrame() {
        return _quickStartIF;
    }

    public TableInternalFrame getLovTableInternalFrame() {
        return _lovTableInternalFrame;
    }

    public HelpNavigatorInternalFrame getHelpNavigatorIF() {
        return _helpNavigatorIF;
    }

    // *************************************************************************
    // *************************** Building Methods ****************************
    // *************************************************************************

    /**
     * Creates and configures the subcomponents.
     */
    private void initComponents() {
        _welcomeIF = new WelcomeInternalFrame();
        _helpContentIF = new HelpContentInternalFrame();
        _lovTableInternalFrame = new TableInternalFrame();
        _lovComboBoxesInternalFrame = new ComboBoxesInternalFrame();
        _performancesInternalFrame = new PerformancesInternalFrame();
        _sqlCustomizationInternalFrame = new SqlCustomizationInternalFrame();

        _quickStartIF = new QuickStartInternalFrame();
        _quickStartIF.setMinimumSize( new Dimension( 192, 225 ) );

        _helpNavigatorIF = new HelpNavigatorInternalFrame();
        _helpNavigatorIF.setMinimumSize( new Dimension( 192, 300 ) );

        Font font = new Font( "dialog", Font.BOLD, 16 );

        _animatedStatusLabel = new BasicTextLabel( " " );
        _animatedStatusLabel.setFont( font );

        _firstStatusLabel = new BasicTextLabel( " " );
        _firstStatusLabel.setFont( font );
        _firstStatusLabel.setBounds( 0, 0, 400, 100 );
        _firstStatusLabel.setOpaque( false );

        _secondStatusLabel = new BasicTextLabel( " " );
        _secondStatusLabel.setFont( font );
        _secondStatusLabel.setBounds( 0, 0, 400, 100 );
        _secondStatusLabel.setOpaque( false );

        _statusGlyphLabel = new GlyphLabel( " ", 5000, 100, DEFAULT_COLOR );
        _statusGlyphLabel.setFont( font );
        _statusGlyphLabel.setBounds( 0, 0, 400, 100 );
        _statusGlyphLabel.setOpaque( false );
    }

    /**
     * Builds this panel with the horizontal <code>JSplitPane</code> in the
     * center and a status bar in the south.
     */
    void build() {
        initComponents();

        setLayout( new BorderLayout() );

        _rightSplitPane = UIFactory.createStrippedSplitPane( JSplitPane.VERTICAL_SPLIT, _welcomeIF, _helpContentIF, 0 );
        _leftSplitPane = UIFactory.createStrippedSplitPane( JSplitPane.VERTICAL_SPLIT, _quickStartIF, _helpNavigatorIF, 0.95 );
        _mainSplitPane = UIFactory.createStrippedSplitPane( JSplitPane.HORIZONTAL_SPLIT, _leftSplitPane, _rightSplitPane, 0 );
        _mainSplitPane.setBorder( BorderFactory.createEmptyBorder( 6, 4, 0, 4 ) );

        _rightSplitPane.setDividerSize( 0 );
        _rightSplitPane.setDividerLocation( 1.0 );
        _rightSplitPane.getBottomComponent().setVisible( false );

        add( _mainSplitPane, BorderLayout.CENTER );
        add( buildStatusBar(), BorderLayout.SOUTH );

        Animator animator = new Animator( createAnimation(), 25 );
        animator.start();
    }

    /**
     * Builds and answers the status bar.
     */
    private JPanel buildStatusBar() {
        FormLayout layout = new FormLayout( "fill:max(200dlu;pref):grow", "fill:30dlu" );

        PanelBuilder builder = new PanelBuilder( layout );
        CellConstraints cc = new CellConstraints();

        builder.add( _firstStatusLabel, cc.xy( 1, 1 ) );
        builder.add( _secondStatusLabel, cc.xy( 1, 1 ) );
        builder.add( _statusGlyphLabel, cc.xy( 1, 1 ) );

        return builder.getPanel();
    }

    // *************************************************************************
    // *************************** Action Methods ******************************
    // *************************************************************************

    private int _storedDividerLocation = 0;

    /**
     * @param p_displayedInternalFrame one of the constant available like
     * WELCOME_INTERNAL_FRAME or LOV_IN_TABLE_INTERNAL_FRAME... If an
     * invalid value the welcome internal frame is displayed.
     */
    public void setDisplayedInternalFrame( int p_displayedInternalFrame ) {
        HelpNode toBeDisplayedNode = null;
        SimpleInternalFrame selectedInternalFrame = null;

        switch( p_displayedInternalFrame ) {
            case ( LOV_IN_TABLE_INTERNAL_FRAME ):
                selectedInternalFrame = _lovTableInternalFrame;
                toBeDisplayedNode = HelpTreeModel.TABLE_WHY_AND_HOW;

                break;
            case ( LOV_IN_DETAIL_INTERNAL_FRAME ):
                selectedInternalFrame = _lovComboBoxesInternalFrame;
                toBeDisplayedNode = HelpTreeModel.DETAIL_WHY_AND_HOW;

                break;
            case ( LOV_CUSTOMIZATION_INTERNAL_FRAME ):
                selectedInternalFrame = _sqlCustomizationInternalFrame;
                toBeDisplayedNode = HelpTreeModel.CUSTOMIZATION_HOW;

                break;
            case ( LOV_PERFORMANCES_INTERNAL_FRAME ):
                selectedInternalFrame = _performancesInternalFrame;
                toBeDisplayedNode = HelpTreeModel.ABOUT_PERFORMANCE;

                break;
            default:
                selectedInternalFrame = _welcomeIF;
        }

        // We set the selected internal frame in the desired JSplitPane (if we have to)
        if ( _rightSplitPane.getTopComponent() != selectedInternalFrame ) {
            if ( _rightSplitPane.getDividerSize() != 0 ) {
                // We keep the divider location for the next use
                _storedDividerLocation = _rightSplitPane.getDividerLocation();
            }

            _rightSplitPane.setTopComponent( selectedInternalFrame );

            // We restore the divider location
            _rightSplitPane.setDividerLocation( _storedDividerLocation );

            // We select the node... The setting of the page in the help
            // content will be then called (since the selection changes,
            // cf. HelpContentInternalFrame)
            _helpNavigatorIF.setSelectedHelpNode( toBeDisplayedNode );
        }
    }

    /**
     * Displays the url of the given help node, if any, in the HelpContentIF.
     *
     * @param p_helpNode The node whose help page must be displayed. Does
     * nothing if null.
     */
    public void setDisplayedHelpNodeInHelpContent( HelpNode p_helpNode ) {
        // We ensure the node is selected
        getHelpNavigatorIF().setSelectedHelpNode( p_helpNode );

        if ( p_helpNode != null && p_helpNode.getURL() != null ) {
            // We display the page if needed
            _helpContentIF.setDisplayedPage( p_helpNode.getURL() );
        }

        // We take care of the divider position
        if ( p_helpNode == null ) {
            // No need to display the help... Divider at the bottom
            _rightSplitPane.setDividerSize( 0 );
            _rightSplitPane.setDividerLocation( 1.0 );
            _rightSplitPane.getBottomComponent().setVisible( false );
        } else {
            // The help content is to be visible...
            _rightSplitPane.getBottomComponent().setVisible( true );

            if ( _rightSplitPane.getDividerSize() == 0 ) {
                // Restoring a default divider location
                if ( _storedDividerLocation == 0 ) {
                    _rightSplitPane.setDividerLocation( _leftSplitPane.getDividerLocation() );
                } else {
                    _rightSplitPane.setDividerLocation( _storedDividerLocation );
                }
            }

            // We restore the default divider size...
            _rightSplitPane.setDividerSize( _leftSplitPane.getDividerSize() );
        }
    }

    // *************************************************************************
    // *************************** Utility Methods *****************************
    // *************************************************************************

    private static final Color DEFAULT_COLOR = new Color( 62, 63, 107 );

    private Animation createAnimation() {
        Animation firstAnimation =
                BasicTextAnimations.defaultFade(
                        _firstStatusLabel,
                        _secondStatusLabel,
                        3000,
                        100,
                        "Why would you use poor web applications...|" +
                        "...when you could benefit from a rich and smart java client ?",
                        DEFAULT_COLOR );

        Animation secondAnimation = new GlyphAnimation( _statusGlyphLabel, 5000, 1, "Bright Side Framework" );

        Animation thirdAnimation =
                BasicTextAnimations.defaultFade(
                        _firstStatusLabel,
                        _secondStatusLabel,
                        3000,
                        100,
                        "Why would you use proprietary buggy systems...|" +
                        "...when Open Source is smart ?",
                        DEFAULT_COLOR );

        Animation fourthAnimation =
                BasicTextAnimation.defaultSpace(
                        _firstStatusLabel,
                        6500,
                        "Bright Side Framework",
                        DEFAULT_COLOR );

        ArrayList animations = new ArrayList();
        animations.add( firstAnimation );
        animations.add( secondAnimation );
        animations.add( thirdAnimation );
        animations.add( fourthAnimation );

        Animation sequential = Animations.sequential( animations );

        Animation result = Animations.repeat( 100f, sequential );
        return Animations.sequential( result, secondAnimation );
    }
}